diff --git a/lib/os_mon/c_src/memsup.c b/lib/os_mon/c_src/memsup.c
index 241e771..dfb26dd 100644
--- a/lib/os_mon/c_src/memsup.c
+++ b/lib/os_mon/c_src/memsup.c
@@ -125,6 +125,10 @@
 #include <sys/sysinfo.h>
 #endif
 
+#if defined (__APPLE__) && defined(__MACH__)
+#include <mach/mach.h>
+#endif
+
 /* commands */
 #include "memsup.h"
 
@@ -440,6 +444,39 @@ get_extended_mem_sgi(memory_ext *me) {
 }
 #endif
 
+#if defined(__APPLE__) && defined(__MACH__)
+static int
+get_extended_mem_darwin(memory_ext *me) {
+    vm_statistics64_data_t vm_stat;
+    mach_port_t myHost;
+    unsigned int count = HOST_VM_INFO64_COUNT;
+    vm_size_t pageSize;
+    uint64_t memTotal, memUsed, memFree;
+
+    myHost = mach_host_self();
+    if (host_page_size(myHost, &pageSize) != KERN_SUCCESS){
+        // vm_stat on darwin does this, so I guess we should too
+        pageSize = 4096;
+    }
+
+    if (host_statistics64(myHost, HOST_VM_INFO64, (host_info64_t)&vm_stat, &count) != KERN_SUCCESS) {
+        return 0;
+    }
+
+    memUsed = (vm_stat.wire_count + vm_stat.active_count + vm_stat.inactive_count);
+    memFree = vm_stat.free_count - vm_stat.speculative_count;
+    memTotal = memUsed + memFree;
+
+    me->total   = memTotal;
+    me->free     = memFree;
+    me->pagesize = pageSize;
+
+    me->flag = F_MEM_TOTAL | F_MEM_FREE;
+
+    return 1;
+}
+#endif
+
 static void
 get_extended_mem(memory_ext *me) {
 /* vxworks */
@@ -458,6 +495,10 @@ get_extended_mem(memory_ext *me) {
 /* sgi */
 #elif defined(sgi) || defined(__sgi) || defined(__sgi__)
     if (get_extended_mem_sgi(me))     return;
+
+/* darwin */
+#elif defined(__APPLE__) && defined(__MACH__)
+    if (get_extended_mem_darwin(me)) return;
 #endif
 
 /* Does this exist on others than Solaris2? */
@@ -522,6 +563,15 @@ fail:
 	print_error("%s", strerror(errno));
 	exit(1); 
     }
+#elif defined(__APPLE__) && defined(__MACH__)
+    memory_ext me;
+    if (get_extended_mem_darwin(&me) < 0) {
+        print_error("Unable to get memory usage.");
+        exit(1);
+    }
+    *tot      = me.total;
+    *pagesize = me.pagesize;
+    *used     = me.total - me.free;
 #else  /* SunOS4 */
     *used = (1<<27);	       	/* Fake! 128 MB used */
     *tot = (1<<28);		/* Fake! 256 MB total */
diff --git a/lib/os_mon/src/memsup.erl b/lib/os_mon/src/memsup.erl
index 4e4eff0..80f4acf 100644
--- a/lib/os_mon/src/memsup.erl
+++ b/lib/os_mon/src/memsup.erl
@@ -174,7 +174,7 @@ init([]) ->
 
     OS = os:type(),
     PortMode = case OS of
-		   {unix, darwin} -> false;
+		   {unix, darwin} -> true;
 		   {unix, freebsd} -> false;
 		   % Linux supports this.
 		   {unix, linux} -> true;
@@ -608,7 +608,7 @@ code_change(Vsn, PrevState, "1.8") ->
 
 	    OS = os:type(),
 	    PortMode = case OS of
-			   {unix, darwin} -> false;
+			   {unix, darwin} -> true;
 			   {unix, freebsd} -> false;
 			   {unix, linux} -> false;
 			   {unix, openbsd} -> true;
@@ -717,23 +717,6 @@ reply(Pending, MemUsage, SysMemUsage) ->
 
 %% get_memory_usage(OS) -> {Alloc, Total}
 
-%% Darwin:
-%% Uses vm_stat command. This appears to lie about the page size in
-%% Mac OS X 10.2.2 - the pages given are based on 4000 bytes, but
-%% the vm_stat command tells us that it is 4096...
-get_memory_usage({unix,darwin}) ->
-    Str = os:cmd("/usr/bin/vm_stat"),
-    {ok, [Free],Str2} = io_lib:fread("Pages free:~d.",skip_to_eol(Str)),
-    {ok, [Active],Str3} =
-	io_lib:fread("Pages active:~d.", skip_to_eol(Str2)),
-    {ok, [Inactive],Str4} =
-	io_lib:fread("Pages inactive:~d.", skip_to_eol(Str3)),
-    {ok, [Wired],_} =
-	io_lib:fread("Pages wired down:~d.", skip_to_eol(Str4)),
-    NMemUsed  = (Wired + Active + Inactive) * 4000,
-    NMemTotal = NMemUsed + Free * 4000,
-    {NMemUsed,NMemTotal};
-
 %% FreeBSD: Look in /usr/include/sys/vmmeter.h for the format of struct
 %% vmmeter
 get_memory_usage({unix,freebsd}) ->
